#include "AISAiboYCrCbColorSeg.h"
#include <string.h>
#include <stdio.h>

AISAiboYCrCbColorSeg::AISAiboYCrCbColorSeg(int width, int height)
: mIAmValid(true),
  mWidth(width),
  mHeight(height),
  mImageSize(width*height),
  mProcessedImage(0),
  mDoubleWidth(width*2),
  mSomeWidth(5*width+1)
{
  mProcessedImage=new unsigned int[mImageSize];
  memset(mYValue, 0, 256*sizeof(unsigned int));
  memset(mCrValue, 0, 256*sizeof(unsigned int));
  memset(mCbValue, 0, 256*sizeof(unsigned int));
}

AISAiboYCrCbColorSeg::AISAiboYCrCbColorSeg(const AISAiboYCrCbColorSeg& other)
  : mIAmValid(other.mIAmValid),
    mWidth(other.mWidth),
    mHeight(other.mHeight),
    mImageSize(mWidth*mHeight),
    mProcessedImage(0),
    mDoubleWidth(other.mDoubleWidth),
    mSomeWidth(other.mSomeWidth)
{
  if(mIAmValid)
    {
      memcpy(mYValue, other.mYValue, sizeof(mYValue));
      memcpy(mCrValue, other.mCrValue, sizeof(mCrValue));
      memcpy(mCbValue, other.mCbValue, sizeof(mCbValue));
      
      mProcessedImage=new unsigned int[mImageSize];
      memcpy(mProcessedImage, other.mProcessedImage, sizeof(unsigned int) * mImageSize);
    }
}


AISAiboYCrCbColorSeg::~AISAiboYCrCbColorSeg(){
  if( (mIAmValid) && (mProcessedImage != 0) )
    delete mProcessedImage;
}

AISAiboYCrCbColorSeg& AISAiboYCrCbColorSeg::operator=(const AISAiboYCrCbColorSeg& other){
  mIAmValid=other.mIAmValid;
  mWidth=other.mWidth;
  mHeight=other.mHeight;
  mImageSize=other.mImageSize;
  
  mDoubleWidth=other.mDoubleWidth;
  mSomeWidth=other.mSomeWidth;

  memcpy(mYValue, other.mYValue, sizeof(mYValue));
  memcpy(mCrValue, other.mCrValue, sizeof(mCrValue));
  memcpy(mCbValue, other.mCbValue, sizeof(mCbValue));
  
  memcpy(mProcessedImage, other.mProcessedImage, sizeof(unsigned int) * mImageSize);
  
  return (*this);
}

const unsigned int* AISAiboYCrCbColorSeg::processImage(unsigned char* pImage){
  if(mIAmValid)
    {
      int line=0;
      int computedPixel;
      unsigned char* ptrSpeedup=pImage-1;
      
      memset(mNumberOfRecognizedPixel, 0, sizeof(int)*32);
      
      for(size_t i=0; i < mImageSize; i++)
	{
	  // Una volta completata una linea
	  if(line == mWidth-1)
	    {
	      ptrSpeedup += mSomeWidth;
	      line =0;
	    }
	  else
	    {
	      ++ptrSpeedup;
	      ++line;
	    }
	  
	  mProcessedImage[i]=computedPixel=
	    (mYValue[ *(ptrSpeedup) ]) & 
	    (mCbValue[ *(ptrSpeedup+mDoubleWidth) ]) & 
	    (mCrValue[ *(ptrSpeedup+mWidth) ]);

	  if(computedPixel != 0)
	    ++mNumberOfRecognizedPixel[ pseudoLog2(computedPixel) ];
	}
      
    }
  return mProcessedImage;
}


bool AISAiboYCrCbColorSeg::setColor(Colors selectedLayer,
				    unsigned char y,
				    unsigned char crMin, unsigned char crMax,
				    unsigned char cbMin, unsigned char cbMax){
  if(mIAmValid)
    {
      //
      // Registro il valore di Y
      //
      mYValue[y]=mYValue[y] | selectedLayer;
      
      
      // Aggiungi controlli di sforamenti.
      for(int i=crMin; i <= crMax; i++)
	{
      mCrValue[i]= mCrValue[i] | selectedLayer;
	}
      
      for(int i=cbMin; i <= cbMax; i++)
	mCbValue[i]= mCbValue[i] | selectedLayer;
      
      return true;    
    }
  else
    {
      return false;
    }
}
