#include "AISYCrCbColorSeg.h"
#include <string.h>
#include <stdio.h>

AISYCrCbColorSeg::AISYCrCbColorSeg(int width, int height)
: mIAmValid(true),
  mWidth(width),
  mHeight(height),
  mImageSize(width*height),
  mProcessedImage(0)
{
  mProcessedImage=new unsigned int[mImageSize];
  memset(mYValue, 0, 256*sizeof(unsigned int));
  memset(mCrValue, 0, 256*sizeof(unsigned int));
  memset(mCbValue, 0, 256*sizeof(unsigned int));
}

AISYCrCbColorSeg::AISYCrCbColorSeg(const AISYCrCbColorSeg& other)
  : mIAmValid(other.mIAmValid),
    mWidth(other.mWidth),
    mHeight(other.mHeight),
    mImageSize(mWidth*mHeight),
    mProcessedImage(0)
{
  if(mIAmValid)
    {
      memcpy(mYValue, other.mYValue, sizeof(mYValue));
      memcpy(mCrValue, other.mCrValue, sizeof(mCrValue));
      memcpy(mCbValue, other.mCbValue, sizeof(mCbValue));
      
      mProcessedImage=new unsigned int[mImageSize];
      memcpy(mProcessedImage, other.mProcessedImage, sizeof(unsigned int) * mImageSize);
    }
}


AISYCrCbColorSeg::~AISYCrCbColorSeg(){
  if( (mIAmValid) && (mProcessedImage != 0) )
    delete mProcessedImage;
}

AISYCrCbColorSeg& AISYCrCbColorSeg::operator=(const AISYCrCbColorSeg& other){
  mIAmValid=other.mIAmValid;
  mWidth=other.mWidth;
  mHeight=other.mHeight;
  mImageSize=other.mImageSize;
  
  memcpy(mYValue, other.mYValue, sizeof(mYValue));
  memcpy(mCrValue, other.mCrValue, sizeof(mCrValue));
  memcpy(mCbValue, other.mCbValue, sizeof(mCbValue));
  
  memcpy(mProcessedImage, other.mProcessedImage, sizeof(unsigned int) * mImageSize);
  
  return (*this);
}

const unsigned int* AISYCrCbColorSeg::processImage(unsigned char* pImage){
  if(mIAmValid)
    {
      int j=-1;
      int computedPixel;
      
      for(size_t i=0; i < mImageSize; i++)
	{
	  mProcessedImage[i]=computedPixel=(mYValue[ pImage[ ++j ] ]) & (mCbValue[ pImage[ ++j ] ]) & (mCrValue[ pImage[ ++j ] ]);
	
	  if(computedPixel != 0)
	    ++mNumberOfRecognizedPixel[ log2(computedPixel) ];  
	}
      
    }
  return mProcessedImage;
}

bool AISYCrCbColorSeg::setColor(Colors selectedLayer,
				unsigned char y,
				unsigned char crMin, unsigned char crMax,
				unsigned char cbMin, unsigned char cbMax){
  if(mIAmValid)
    {
      //
      // Registro il valore di Y
      //
      mYValue[y]=mYValue[y] | selectedLayer;
      
      
      // Aggiungi controlli di sforamenti.
      for(int i=crMin; i <= crMax; i++)
	{
	  mCrValue[i]= mCrValue[i] | selectedLayer;
	}
      
      for(int i=cbMin; i <= cbMax; i++)
	mCbValue[i]= mCbValue[i] | selectedLayer;
      return true;    
    }
  else
    {
      return false;
    }
}
